/*
 * kmerlin.s
 * Ccopyright (c) 2003-2004, deitest@hotmail.com, yanmar@ogonigo.com
 * 
 *kmerlin.s is based on  kmerlin ver1.41. olueg@olsd.de
 *Fixed by deitest@hotmail.com, yanmar@ogonigo.com 2003-2004
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "logviewimpl.h"
#include "kmsMultiLineEdit.h"
#include "kmerlin.h"
#include "onlineview.h"
#include "mylistview.h"

#include <qmultilineedit.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qaccel.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qlabel.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <qevent.h>
#include <qfont.h>

logviewimpl::logviewimpl( QWidget* parent,  const char* name, QString _infoFontSize, QString _menuFontSize, WFlags fl )
    : logview( parent, name, _infoFontSize, _menuFontSize,fl )
{
	fnPress = false;
    connect( logView, SIGNAL(keyPress(QKeyEvent *)), this, SLOT(keypressed(QKeyEvent *)) );
    connect( logView, SIGNAL(keyRelease(QKeyEvent *)), this, SLOT(keyreleased(QKeyEvent *)) );
	logView->setWordWrap(QMultiLineEdit::NoWrap);
}

void logviewimpl::init()
{
	bool ok;
	QFont f = logView->font();
	f.setPointSize((theApp->_logFontSize).toInt(&ok,10));
	f.setFamily(theApp->_fontName);
	logView->setFont(f);

  logClose = new QAction(tr("Close"), tr("C&lose"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  logClose->setStatusTip(tr("Close the Tab"));
  logClose->setWhatsThis(tr("Close\n\nClose the LogView"));
  connect(logClose, SIGNAL(activated()), this, SLOT(slotLogClose()));

  editCopy = new QAction(tr("Copy"), tr("&Copy"), QAccel::stringToKey(tr("Ctrl+C")), this);
  editCopy->setStatusTip(tr("Copies the selected section to the clipboard"));
  editCopy->setWhatsThis(tr("Copy\n\nCopies the selected section to the clipboard"));
  connect(editCopy, SIGNAL(activated()), this, SLOT(slotEditCopy()));

  logDelete = new QAction(tr("Delete Log"), tr("&Delete Log"), 0, this);
  logDelete->setStatusTip(tr("Delete Log"));
  logDelete->setWhatsThis(tr("Delete Log\n\nDelete the Log file"));
  connect(logDelete, SIGNAL(activated()), this, SLOT(slotLogDelete()));

  popMenu=new QPopupMenu();

  editCopy->addTo(popMenu);
  popMenu->insertSeparator();

  logDelete->addTo(popMenu);
  popMenu->insertSeparator();

  logClose->addTo(popMenu);

	infoText->setText("Conversation Log with: " + chatWith);

	loadLog();
}

logviewimpl::~logviewimpl()
{
}

void logviewimpl::slotLogClose(){
	theApp->logList.remove(this);
    theApp->view->TabView->removePage(this);
    theApp->view->ListView->setFocus();    
	delete this;
}

void logviewimpl::slotTabChange(int direction)
{

	theApp->changeTab(direction);
}

void logviewimpl::slotLogDelete(){
    QDir dir;
	dir.setPath(theApp->_logPath);
    if ( !dir.exists() ) mkdir(dir.path().local8Bit(),0700);
	dir.setPath(theApp->_logPath + "/KMerlin_Log");
    if ( !dir.exists() ) mkdir(dir.path().local8Bit(),0700);
    QString url = dir.path() + "/" + chatWith + ".txt";

	QFile f(url);
	if(f.exists()){
		f.remove();
	}

	logView->setText("");
	slotLogClose();
}

void logviewimpl::slotShowMenu()
{
	popMenu->exec(menuBtn->mapToGlobal(QPoint(0,0)));
}

void logviewimpl::slotCloseTab()
{
    slotLogClose();
}


/** No descriptions */
void logviewimpl::slotEditCopy()
{
	if(logView->hasFocus())
	{
		logView->copy();
	}
}

void logviewimpl::keypressed(QKeyEvent * k)
{
    bool s = (k->state() & ShiftButton);
    bool a = (k->state() & AltButton);
    bool c = (k->state() & ControlButton);
    switch (k->key()) {
        case 0x201a          : theApp->ListViewSetFocus();            break;
        case Key_PageUp      : slotTabChange(-1);		        break;
        case Key_PageDown    : slotTabChange(1);		        break;
        case Key_Home        : logView->cursorHome(s);		        break;
        case Key_End         : logView->cursorEnd(s);			break;
	case Key_F22         : fnPress = true;       											break;
//        case Key_X           : if(a || c || fnPress) logView->cut();							break;
        case Key_C           : if(a || c || fnPress) logView->copy(); 							break;
//        case Key_V           : if(a || c || fnPress) logView->paste();						break;
        case Key_Left        : if(a || c || fnPress) slotTabChange(-1); else logView->cursorL(s);		        break;
        case Key_Right       : if(a || c || fnPress) slotTabChange(1); else logView->cursorR(s);			break;
//        case Key_Left        : logView->cursorL(s);												break;
//        case Key_Right       : logView->cursorR(s);												break;
        case Key_Up          : logView->cursorU(s);       										break;
        case Key_Down        : logView->cursorD(s);    											break;
        case Key_Tab         : if(s) slotShowMenu(); else theApp->view->TabView->setFocus();  	break;
		case Key_Escape      : slotCloseTab();												   	break;

    }
}

void logviewimpl::keyreleased(QKeyEvent * k)
{
    switch (k->key()) {
	case Key_F22         : fnPress = false;       									break;
    }
}

void logviewimpl::slotRePaint()
{
	theApp->view->TabView->showPage(this);
}

void logviewimpl::loadLog(){
	
	logView->setText("");

    QDir dir;
	dir.setPath(theApp->_logPath);
    if ( !dir.exists() ) mkdir(dir.path().local8Bit(),0700);
	dir.setPath(theApp->_logPath + "/KMerlin_Log");
    if ( !dir.exists() ) mkdir(dir.path().local8Bit(),0700);
    QString url = dir.path() + "/" + chatWith + ".txt";

	QFile f(url);
	bool ok = false;
	if(f.exists()){
		ok = f.open(IO_ReadOnly);
	}

	if(ok) {
		QTextStream stream( &f );
		f.flush();
		QString log = stream.read();
		f.close();

		logView->setText(log);
		logView->setCursorPosition(logView->numLines(),0,false);

	}
}

